"""Download ERA-5 data from the Copernicus Climate Data Store."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

# builtins
import os
import pathlib
from joblib import Parallel, delayed

# externals
import cdsapi
import numpy as np

# ERA-5 product
product = 'reanalysis-era5-pressure-levels'
product_type = 'reanalysis'

# pressure levels
pressure_levels = ['850', '500']

# variables
variables = ['geopotential', 'temperature', 'u_component_of_wind',
             'v_component_of_wind', 'specific_humidity']

# time period
years = [str(y) for y in np.arange(1981, 2011)]
month = [str(m) for m in np.arange(1, 13)]
days = [str(d) for d in np.arange(1, 31)]
time = ["{:02d}:00".format(t) for t in np.arange(0,24)]

# area of interest (Alps): North, West, South, East
area = [52, 2, 40, 20]

# ERA5 download configuration dictionary
CONFIG = {
    'product_type': product_type,
    'pressure_level': pressure_levels,
    'month': month,
    'day': days,
    'time': time,
    'format': 'netcdf',
    'area': area
}

# output path
target = pathlib.Path('/mnt/CEPH_PROJECTS/FACT_CLIMAX/REANALYSIS/ERA5/')


if __name__ == '__main__':

    # initialize client
    c = cdsapi.Client()

    # download data for the different variables
    for var in variables:
        # create output directory
        output = target.joinpath(var)
        if not output.exists():
            output.mkdir(parents=True, exist_ok=True)

        # split the download to the different years: CDS API cannot handle
        # requests over the whole time period
        Parallel(n_jobs=min(len(years), os.cpu_count()), verbose=51)(
            delayed(c.retrieve)(
                product,
                {**CONFIG, **{'variable': var, 'year': year}},
                output.joinpath('_'.join(['ERA5', var, year]) + '.nc'))
            for year in years)
